import os
import unittest

from typer.testing import CliRunner  # type: ignore

from symeo_python.cli.cli import CliPort
from symeo_python.cli.symeo_python_cli import (
    DEFAULT_CONFIG_FORMAT_PATH,
    SymeoPythonCli,
    DEFAULT_LOCAL_CONFIG_PATH,
    DEFAULT_SYMEO_API_URL,
)


class MainTest(unittest.TestCase):
    __current_absolute_path = os.path.dirname(os.path.abspath(__file__))

    def test_should_run_cli_build_command(self):
        # Given
        cli_adapter_mock = CliAdapterMock()
        cli = SymeoPythonCli(cli_adapter_mock).load_commands()
        runner = CliRunner()

        # When
        result = runner.invoke(cli, "build")

        # Then

        self.assertTrue(
            cli_adapter_mock.config_contract.endswith(
                DEFAULT_CONFIG_FORMAT_PATH.replace("./", "")
            )
        )
        self.assertEqual(0, result.exit_code)

    def test_should_run_cli_start_command_in_local_mode(self):
        # Given
        cli_adapter_mock = CliAdapterMock()
        cli = SymeoPythonCli(cli_adapter_mock).load_commands()
        runner = CliRunner()
        fake_sub_process = "fake sub_process"

        # When
        result = runner.invoke(cli, "start -- %s" % fake_sub_process)

        # Then
        self.assertEqual(0, result.exit_code)
        self.assertEqual(
            fake_sub_process.split(" "), cli_adapter_mock.cli_input_data["sub_process"]
        )
        self.assertTrue(
            str(cli_adapter_mock.cli_input_data["config_values_path"]).endswith(
                DEFAULT_LOCAL_CONFIG_PATH.replace("./", "")
            )
        )

    def test_should_run_cli_start_command_in_saas_mode(self):
        # Given
        cli_adapter_mock = CliAdapterMock()
        cli = SymeoPythonCli(cli_adapter_mock).load_commands()
        runner = CliRunner()
        fake_sub_process = "fake sub_process"
        fake_api_key = "API_111_222"

        # When
        result = runner.invoke(
            cli, "start -k %s -- %s" % (fake_api_key, fake_sub_process)
        )

        # Then
        self.assertEqual(0, result.exit_code)
        self.assertEqual(
            fake_sub_process.split(" "), cli_adapter_mock.cli_input_data["sub_process"]
        )
        self.assertTrue(
            str(cli_adapter_mock.cli_input_data["config_values_path"]).endswith(
                DEFAULT_LOCAL_CONFIG_PATH.replace("./", "")
            )
        )
        self.assertEqual(cli_adapter_mock.cli_input_data["api_key"], fake_api_key)
        self.assertEqual(
            cli_adapter_mock.cli_input_data["api_url"], DEFAULT_SYMEO_API_URL
        )

    def test_should_run_cli_start_command_in_saas_mode_given_an_api_url(self):
        # Given
        cli_adapter_mock = CliAdapterMock()
        cli = SymeoPythonCli(cli_adapter_mock).load_commands()
        runner = CliRunner()
        fake_sub_process = "fake sub_process"
        fake_api_key = "API_111_222"
        fake_api_url = "http://fake-url.io"

        # When
        result = runner.invoke(
            cli,
            "start -k %s -a %s -- %s" % (fake_api_key, fake_api_url, fake_sub_process),
        )

        # Then
        self.assertEqual(0, result.exit_code)
        self.assertEqual(
            fake_sub_process.split(" "), cli_adapter_mock.cli_input_data["sub_process"]
        )
        self.assertTrue(
            str(cli_adapter_mock.cli_input_data["config_values_path"]).endswith(
                DEFAULT_LOCAL_CONFIG_PATH.replace("./", "")
            )
        )
        self.assertEqual(cli_adapter_mock.cli_input_data["api_key"], fake_api_key)
        self.assertEqual(cli_adapter_mock.cli_input_data["api_url"], fake_api_url)


class CliAdapterMock(CliPort):

    config_contract: str
    cli_input_data: dict

    def generate_configuration_from_contract_file(self, config_contract: str):
        self.config_contract = config_contract

    def prepare_env_and_start_sub_process(self, cli_input_data: dict):
        self.cli_input_data = cli_input_data



import os
from pathlib import Path
from typing import List

from typer import Option, Typer, get_app_dir  # type: ignore

from symeo_python.cli.cli import CliPort

DEFAULT_CONFIG_FORMAT_PATH = "./symeo.config.yml"
DEFAULT_LOCAL_CONFIG_PATH = "./symeo.local.yml"
DEFAULT_SYMEO_API_URL = "https://config-staging.symeo.io/api/v1/values"


class SymeoPythonCli:
    __cli_port: CliPort

    def __init__(self, cli_port: CliPort):
        self.__cli_port = cli_port

    def load_commands(
        self,
    ) -> Typer:
        cli = Typer()

        @cli.command()
        def build(
            config_contract: str = Option(
                DEFAULT_CONFIG_FORMAT_PATH, "--config-contract", "-c"
            )
        ):
            config_contract_path: Path = Path(os.getcwd()) / config_contract
            print(
                f"Starting to generate Config class from configuration contract {config_contract_path}"
            )
            self.__cli_port.generate_configuration_from_contract_file(
                str(config_contract_path)
            )

        @cli.command()
        def start(
            sub_process: List[str],
            api_key: str = Option("", "--api-key", "-k"),
            api_url: str = Option(DEFAULT_SYMEO_API_URL, "--api-url", "-a"),
            config_values: str = Option(
                DEFAULT_LOCAL_CONFIG_PATH, "--config-values", "-f"
            ),
        ):
            config_values_path: Path = Path(os.getcwd()) / config_values
            cli_input_data = {
                "config_values_path": str(config_values_path),
                "sub_process": sub_process,
            }
            if api_key != "":
                cli_input_data["api_key"] = api_key
                cli_input_data["api_url"] = api_url
            self.__cli_port.prepare_env_and_start_sub_process(cli_input_data)

        return cli



from symeo_python.configuration.config_loader import ConfigLoaderAdapter
from symeo_python.configuration.configuration import Config
from symeo_python.api_client.symeo_api_client import SymeoApiClientAdapter

config: Config = ConfigLoaderAdapter(SymeoApiClientAdapter()).load_config_from_env()



import requests

SYMEO_API_KEY_HEADER = "X-API-KEY"


class SymeoApiClientPort:
    def get_conf_values_for_api_key(self, api_url: str, api_key: str) -> dict:
        pass


class SymeoApiClientAdapter(SymeoApiClientPort):
    def get_conf_values_for_api_key(self, api_url, api_key):
        response = requests.get(url=api_url, headers={SYMEO_API_KEY_HEADER: api_key})
        if response.status_code != 200:
            raise Exception(
                f"Http status {response.status_code} - Failed to fetch configuration values from Symeo API {api_url}"
            )
        return response.json()["values"]
